@@ -40,16 +40,22 @@ EMPTY_SHA256_HASH = (
     'e3b0c44298fc1c149afbf4c8996fb92427ae41e4649b934ca495991b7852b855')
 
 
-class SigV2Auth(object):
+class BaseSigner(object):
+    REQUIRES_REGION = False
+
+    def add_auth(self, request):
+        raise NotImplementedError("add_auth")
+
+
+class SigV2Auth(BaseSigner):
     """
     Sign a request with Signature V2.
     """
-    def __init__(self, credentials, service_name=None, region_name=None):
+
+    def __init__(self, credentials):
         self.credentials = credentials
         if self.credentials is None:
             raise NoCredentialsError
-        self.service_name = service_name
-        self.region_name = region_name
 
     def calc_signature(self, request, params):
         logger.debug("Calculating signature using v2 auth.")
@@ -97,13 +103,11 @@ class SigV2Auth(object):
         return request
 
 
-class SigV3Auth(object):
-    def __init__(self, credentials, service_name=None, region_name=None):
+class SigV3Auth(BaseSigner):
+    def __init__(self, credentials):
         self.credentials = credentials
         if self.credentials is None:
             raise NoCredentialsError
-        self.service_name = service_name
-        self.region_name = region_name
 
     def add_auth(self, request):
         if 'Date' not in request.headers:
@@ -120,19 +124,20 @@ class SigV3Auth(object):
         request.headers['X-Amzn-Authorization'] = signature
 
 
-class SigV4Auth(object):
+class SigV4Auth(BaseSigner):
     """
     Sign a request with Signature V4.
     """
+    REQUIRES_REGION = True
 
-    def __init__(self, credentials, service_name=None, region_name=None):
+    def __init__(self, credentials, service_name, region_name):
         self.credentials = credentials
         if self.credentials is None:
             raise NoCredentialsError
         self.now = datetime.datetime.utcnow()
         self.timestamp = self.now.strftime('%Y%m%dT%H%M%SZ')
-        self.region_name = region_name
-        self.service_name = service_name
+        self._region_name = region_name
+        self._service_name = service_name
 
     def _sign(self, key, msg, hex=False):
         if hex:
@@ -208,16 +213,16 @@ class SigV4Auth(object):
     def scope(self, args):
         scope = [self.credentials.access_key]
         scope.append(self.timestamp[0:8])
-        scope.append(self.region_name)
-        scope.append(self.service_name)
+        scope.append(self._region_name)
+        scope.append(self._service_name)
         scope.append('aws4_request')
         return '/'.join(scope)
 
     def credential_scope(self, args):
         scope = []
         scope.append(self.timestamp[0:8])
-        scope.append(self.region_name)
-        scope.append(self.service_name)
+        scope.append(self._region_name)
+        scope.append(self._service_name)
         scope.append('aws4_request')
         return '/'.join(scope)
 
@@ -237,8 +242,8 @@ class SigV4Auth(object):
         key = self.credentials.secret_key
         k_date = self._sign(('AWS4' + key).encode('utf-8'),
                             self.timestamp[0:8])
-        k_region = self._sign(k_date, self.region_name)
-        k_service = self._sign(k_region, self.service_name)
+        k_region = self._sign(k_date, self._region_name)
+        k_service = self._sign(k_region, self._service_name)
         k_signing = self._sign(k_service, 'aws4_request')
         return self._sign(k_signing, string_to_sign, hex=True)
 
@@ -267,7 +272,7 @@ class SigV4Auth(object):
         return request
 
 
-class HmacV1Auth(object):
+class HmacV1Auth(BaseSigner):
 
     # List of Query String Arguments of Interest
     QSAOfInterest = ['acl', 'cors', 'defaultObjectAcl', 'location', 'logging',
@@ -283,8 +288,6 @@ class HmacV1Auth(object):
         self.credentials = credentials
         if self.credentials is None:
             raise NoCredentialsError
-        self.service_name = service_name
-        self.region_name = region_name
         self.auth_path = None  # see comment in canonical_resource below
 
     def sign_string(self, string_to_sign):
