@@ -21,6 +21,8 @@
 # IN THE SOFTWARE.
 #
 import logging
+
+import six
 from requests import models
 from requests.sessions import REDIRECT_STATI
 
@@ -83,14 +85,27 @@ class AWSPreparedRequest(models.PreparedRequest):
     def __init__(self, original_request):
         self.original = original_request
         super(AWSPreparedRequest, self).__init__()
-        self.hooks.setdefault('response', []).append(self.reset_stream)
+        self.hooks.setdefault('response', []).append(
+            self.reset_stream_on_redirect)
 
-    def reset_stream(self, response, **kwargs):
+    def reset_stream_on_redirect(self, response, **kwargs):
         if response.status_code in REDIRECT_STATI and \
                 isinstance(self.body, file_type):
             logger.debug("Redirect received, rewinding stream: %s", self.body)
-            try:
-                self.body.seek(0)
-            except Exception as e:
-                logger.debug("Unable to rewind stream: %s", e)
-                raise UnseekableStreamError(stream_object=self.body)
+            self.reset_stream()
+
+    def reset_stream(self):
+        # Trying to reset a stream when there is a no stream will
+        # just immediately return.  It's not an error, it will produce
+        # the same result as if we had actually reset the stream (we'll send
+        # the entire body contents again if we need to).
+        # Same case if the body is a string/bytes type.
+        if self.body is None or isinstance(self.body, six.text_type) or \
+                isinstance(self.body, six.binary_type):
+            return
+        try:
+            logger.debug("Rewinding stream: %s", self.body)
+            self.body.seek(0)
+        except Exception as e:
+            logger.debug("Unable to rewind stream: %s", e)
+            raise UnseekableStreamError(stream_object=self.body)
